#include "appli.h"

DWORD WINAPI ComManga(char *Directory)
{
	char CommandLine[384],buff[384],Path[384];
	WIN32_FIND_DATA FileData;					//Necessary to ensure the validity of the folder (to be converted into CBZ) information
	STARTUPINFO StartupInfo;					//Containing informations necessary for the calling of ComManga process
	PROCESS_INFORMATION ProcessInformation;		//Containing informations necessary for the calling and ending of ComManga process
	DWORD ProcessExitCode;						//Necessary to ensure the running state of the ComManga process
	
	ZeroMemory(buff,sizeof buff);
	ZeroMemory(Path,sizeof Path);
	
	//Copying the argument in a buffer is the only way I found to correct a problem that occurred after calling ComManga() in a thread :
	//The argument (Directory) was only readable once. I checked the values of it at the end and at the beginning of ComManga, but each
	//time, the data was lost after being read.
	//The "static" from the caller side made things worst, so I went for a simple copy in a function-local buffer.
	sprintf(Path,"%s",Directory);
	
	// Check consistency of the Path
	if(!PathIsDirectory(Path)) //if Path is erroneous
	{
		sprintf(buff,"ERROR : %s : invalid path. Please check it and try again.",Path);
		WriteToLB(buff);
		return 0;
	}
	else //if Path is valid
	{
		// Preparing usage of CreateProcess function - this is CRUCIAL since the function will fail if the 2 structures are not properly filled before usage
		ZeroMemory(CommandLine,sizeof CommandLine);
		memset(&StartupInfo,0,sizeof(StartupInfo));
		memset(&ProcessInformation,0,sizeof(ProcessInformation));
		StartupInfo.cb = sizeof(StartupInfo);
		//Configuring the  StartupInfo structure in order to hide the ComManga process
		//The reason is when you start a series of mangas, if you continue to use your computer, each ComManga launch
		//will pop up and make you lose the focus of your current window.
		StartupInfo.dwFlags=STARTF_USESHOWWINDOW;
		StartupInfo.wShowWindow=SW_HIDE;
		
		// Preparing command line
		sprintf(CommandLine,"ComManga -d \"%s\" -n \"%s\"",Path,Path);

		// CreateProcess - this function is complex but allows to open, monitor and close processes later on
		if(CreateProcess(
			NULL,
			CommandLine,
			NULL,
			NULL,
			false,
			0,
			NULL,
			NULL,
			&StartupInfo,
			&ProcessInformation)==0) // If CreateProcess failed
		{
			WriteToLB("Could not start ComManga. Please check you have all the files in the ComMangaGUI folder.");
		}
		else //if CreateProcess() succeeded
		{
			sprintf(buff,"CBZ creation in progress... (%s.cbz)",GetLastFolder(Path));
			WriteToLB(buff);
		}

		// If CreateProcess worked, we wait for the end of ComManga
		for(;;)
		{
			// Checking state of ComManga using the handle returned by CreateProcess
			GetExitCodeProcess(ProcessInformation.hProcess,&ProcessExitCode);
			// If Exit code if other than active, exit the loop
			if(ProcessExitCode!=STILL_ACTIVE) break;
			// This Sleep for 50ms allows to not take up too much resources
			Sleep(50);
		}
		
		//Checking file existence
		sprintf(buff,"%s.cbz",Path);
		if(FindFirstFile(buff,&FileData)==INVALID_HANDLE_VALUE)
		{
			WriteToLB("ERROR : CBZ could not be found. Please check write autorization on the folder.");
		}
		else
		{
			sprintf(Path,"CBZ created successfully! %s",buff);
			WriteToLB(Path);
		}

		//Closing process handle (no big deal if not done)
		CloseHandle(ProcessInformation.hProcess);
		CloseHandle(ProcessInformation.hThread);
		
	}
	return 1;
}